import numpy as np
import pacs
import math
import formattimeints as ftint
import linreg
import carmatable as ct
import colorsys
import os

def colors(n):
    """
    returns n colors as seperated as possible.
    """
    step=1.0/(n-1)
    output=[(0,0,0)]
    for i in range(n):
        output.append(colorsys.hsv_to_rgb(i*step,1,1))
    return output
def normalize(input):
    """
    Normalizes the value to betwen 0 and 1"""
    maxF=float(max(input))
    minF=float(min(input))
    return ([(float(x)-minF)/(maxF-minF) for x in input],minF,maxF)
def getTimes(file):
    """
    The purpose of this function is to take a "timeops.py" output and return the starttimes,
    endtimes, and sources as a tuple of lists.
    """
    import carmatable as ct
    tableIn = ct.CarmaTable()
    tableIn.open(file)
    starttimes = tableIn.getCol(0)
    endtimes = tableIn.getCol(1)
    sources = tableIn.getCol(2)
    starttimes = ftint.addColons(starttimes)
    endtimes = ftint.addColons(endtimes)
    return (starttimes, endtimes, sources)
def midnight(times):
    """
        The purpose of this function is to take a list of times and return a list of times that have all the over-midnight times
    with added 24 hours.    
    """
    newTime = [times[0]]
    for time in times[1:]:
        if time<newTime[-1]:
            newTime.append(str((int(time[0:2])+24))+time[2:])
        else:
            newTime.append(time)
    return newTime
def paralell(func,argsList,kwargsList=None):
    """
    This function will execute func once on every element in argsList and kwargsList
    in seperate processes. The lists must be lists of lists, even if the function only
    takes 1 arguemnt. If it take no arguments, a list of empty lists of appropriate
    length should be provided.
    """
    if not kwargsList:
        kwargsList=[[]]*len(argsList)
    for arg,kwarg in zip(argsList,kwargsList):
        pid=os.fork()
        if pid==0:#The child
            if kwarg:
                func(*argsList[0],**kwargsList[0])
            else:
                func(*argsList[0])
            os._exit(os.EX_OK)
        else:#The parent
            argsList=argsList[1:]
            kwargsList=kwargsList[1:]
    while True:
        try:
            os.wait()
        except OSError:
            break
def linregLists(xList,yList,method=3):
    """
    This program takes in 2 lists and returns
    data that is useful for plotting the
    fit line.
    Three items are returned:
        line-The x and y co-ordinates of a line
        of best fit.
        envelope- The x and y co-ordinates of
        an error envelope arround the line of
        best fit.
        fit-The raw output of linreg.
    """
    toFloat=lambda x:[float(y) for y in x]
    xList=toFloat(xList)
    yList=toFloat(yList)
    fit = linreg.linreg(xList,yList,method)
    slopeH=fit[0]+fit[2]
    slopeL=fit[0]-fit[2]
    intH=fit[1]+fit[3]
    intL=fit[1]-fit[3]
    xMin=np.min(xList)
    xMax=np.max(xList)
    linex=np.array([xMin,xMax])
    liney=(linex*float(fit[0]))+float(fit[1])
    if xMin<0<xMax:
        envelopex=[xMin,0,xMax,xMax,0,xMin]
        envelopey=[slopeL*xMin+intH,intH,slopeH*xMax+intH,slopeL*xMax+intL,intL,slopeH*xMin+intL]
        return ((linex,liney),(envelopex,envelopey),fit)
    else:
        envelopex=[xMin,xMax,xMax,xMin]
        envelopey=[slopeL*xMin+intL,slopeL*xMax+intL,slopeH*xMax+intH,slopeH*xMin+intH]
        return ((linex,liney),(envelopex,envelopey),fit)
   
def getSources(file):
    """The purpose of this function is to take a "timeops.py" output and return a list of sources"""
    import carmatable as ct
    tableIn = ct.CarmaTable()
    tableIn.open(file)
    sources = tableIn.getCol(2)
    return sources

def timeToSec(time):
    """
        The purpose of this function is to take a H:M:S time and convert it into seconds.
    """
    seconds = (pacs.sexa2dec(time) * 3600)
    return seconds

def stdev(list):
    """takes a list of numbers and returns the standard deviation of them"""
    a = np.array(list)
    return a.std()

def avg(list):
    """takes a list of numbers and returns the average of them"""
    sum = 0
    for element in list:
        sum = sum + float(element)
    average = (sum/len(list))
    return average
def baselineLength(baseline, miriadfile,antposfile=None):
    """
        The purpose of this function is to take a baseline and a miriadfile and output the length of the baseline.
    Input:
    baseline: A string that has two antenna numbers separated by a "-". For example: "1-2" is the baseline string for the baseline
    between antenna 1 and antenna 2.
    miriadfile: The file path of the miriad file for the baseline.
    antposfile: The antenna position file for the miriad file of the baseline

    Output:
    distance: The distance between the two antennae of the baseline input string
    """

    antlist = baseline.split('-')
    ant1 = int(antlist[0])
    ant2 = int(antlist[1])
    #This will find the index of the antenna in the listobs output
    if antposfile == None:
        import enulist as enu
        coords = enu.enulist(miriadfile)
	# print 'no antposfile:',ant1,ant2
        east1  = coords[ant1][0]
        east2  = coords[ant2][0]
        north1 = coords[ant1][1]
        north2 = coords[ant2][1]
    else:
        # use new xyz2enu
        import xyz2enu
        xyz = xyz2enu.read_antpos(antposfile)
        enu = xyz2enu.xyz2enu(xyz)

        east1 = enu[ant1 - 1][0]
        east2 = enu[ant2 - 1][0]
        north1 = enu[ant1 - 1][1]
        north2 = enu[ant2 - 1][1]
    distance = math.sqrt((east1-east2)**2 + (north1-north2)**2)
    return distance

def calcRMSPath(starttimes,endtimes):
    fin = open("rmspath.txt", 'r')
    lines = fin.readlines()
    fin.close()
    times = []
    rmslist = []
    #changesTimes = []
    #changesRMS = []
    for line in lines:
        if line[0] == "#": continue
        words = line.split()
        times.append(words[1])
        rmslist.append(words[2])
    #read in data into lists: times, rmslist

    #count = 0
    #for (time,rms) in zip(times,rmslist):
    #    if count > 0:
    #        if str(rms) != str(oldRMS):
    #            changesTimes.append(time)
    #            changesRMS.append(rms)
    #    else:
    #        changesTimes.append(time)
    #        changesRMS.append(rms)
    #    oldRMS = rms
    #    count = count + 1

    names = ["starttimes","endtimes", "changesTimes", "changesRMS"]
    types = ["","","",""]
    units = ["","","",""]
    tableOut = ct.CarmaTable(names,types,units)
    
    outstarts = []
    outends = []
    outtimes = []
    outrmslist = []
    
    for (starttime,endtime) in zip(starttimes,endtimes):
        #for (time,rms) in zip(changesTimes,changesRMS):
        for (time,rms) in zip(times,rmslist):
            if timeToSec(starttime) < timeToSec(time) < timeToSec(endtime):
                #tableOut.addRow([starttime,endtime,time,rms])
                outstarts.append(starttime)
                outends.append(endtime)
                outtimes.append(time)
                outrmslist.append(rms)

    
    #tableOut.write("superALLtimestest.time")
    return (outstarts,outends,outtimes,outrmslist)
    
    #print changesTimes
    #print
    #print changesRMS

def calcWeatherVar(starttimes,endtimes,varName):
    fin = open("%s.txt"%(varName), 'r')
    lines = fin.readlines()
    fin.close()
    times = []
    varlist = []
    changesTimes = []
    changesVar = []
    for line in lines:
        if line[0] == "#": continue
        words = line.split()
        times.append(words[1])
        varlist.append(words[2])
    #read in data into lists: times, rmslist

    names = ["starttimes","endtimes", "changesTimes", "changesRMS"]
    types = ["","","",""]
    units = ["","","",""]
    tableOut = ct.CarmaTable(names,types,units)
    
    outstarts = []
    outends = []
    outtimes = []
    outvarlist = []

    newTime = [times[0]]
    for time in times[1:]:
        if time<newTime[-1]:
            newTime.append(str((int(time[0:2])+24))+time[2:])
        else:
            newTime.append(time)
    #print times
    times = newTime[:]
    #print times
    
    for (starttime,endtime) in zip(starttimes,endtimes):
        #for (time,var) in zip(changesTimes,changesVar):
        for (time,var) in zip(times,varlist):
            if starttime < timeToSec(time) < endtime:
                #tableOut.addRow([starttime,endtime,time,rms])
                outstarts.append(starttime)
                outends.append(endtime)
                outtimes.append(time)
                outvarlist.append(var)

    return (outstarts,outends,outtimes,outvarlist)
